/************************************************//** @file HepMCVisualFilter.h 
*
* Header file for the filter and helper class.
* VisualFilter is a filter class with GUI window for HepMC GenEvents    
* Basic functions include: Vertex finding, particle and vertex slimming.
*
*
* @author <A HREF="mailto:sboeser@hep.ucl.ac.uk">Sebastian B&ouml;ser</A>
* @date 06.08.07
*
*//****************************************************************************/

/*****************/
/*      ToDo     */
/*****************/
//
// The GUI has not been implemented yet. The main problem is that GUI
// implementation w/o QObject signal/slot mechanism is a pain in the ass.
// However, rootcint - which is needed for signal/slot is not compatible with
// HepMC classes in ROOT version < 5.16 and HepMC versions < 2.0.0.5 .
// Therefore, dictionaries for all HepMC classes are currently made available
// via genreflex dictionaries. For a GUI for the HepMC::VisualFilter, this means
// means we need to implement an interface class, that can be compiled with
// genreflex and rootcint, to interface the rootcinted GUI to the genreflex
// HepMC::VisualFilter.
//
// For the use with HepMC browser, and independant GUI has been written in
// python.

//-- STL libraries
#include <vector>
//#include <algorithm>  // Include algorithms, such as sort

//-- ROOT headers
#include "Riostream.h"
#include "Rtypes.h"

//-- HepMC headers
//#include "HepMC/ParticleDataTable.h"
//#include "HepMC/ParticleData.h"
#include "HepMC/GenParticle.h"
#include "HepMC/GenVertex.h"
#include "HepMC/GenEvent.h"

//Add our classes to HepMC namespace
namespace HepMC {


///Helper class to prepare and filter events. Although completely independant,
///the intention of this class is for the use with the HepMCVisual graphics
///elements.Two main purposes are served with
///this class:
/// - slim events, e.g. by removing particles from GEANT simulation
/// - find and set the signal process vertex (i.e. the first vertex drawn by
///   VisualEvent::Draw())
class VisualFilter {

  public:

    ///Constructor
    VisualFilter();
    
    ///default destructor
    ~VisualFilter();

    ///Find signal process by in- and out-particle list.
    ///The \c in and \c out vectors are lists of particle IDs. Every vertex in
    ///the event is checked if it has \em at \em least these particles as
    ///incoming
    ///and outgoing particles and is then set as signal vertex of the event.
    ///The function returns \c TRUE if a vertex is found, otherwise \c FALSE
    Bool_t FindSignalProcess( HepMC::GenEvent& event,
                              const std::vector<Int_t> in,
                              const std::vector<Int_t> out );

    ///Remove all GEANT particles and vertices. Particles added to the event by
    ///the GEANT simulation are identified by having a barcode that is larger
    ///than 200000.
    ///\bug This methods has been observed causing errors, probably due to
    ///improperly removed particles, for which a reference is still held by the
    ///owning vertex.
    void RemoveGeant( HepMC::GenEvent& event );

    ///Remove pythia hadronization step.
    ///In the hadronization, pythia collects all relevant particles in groups
    ///(e.g. all colour-connected particles) and then adds a vertex with the
    ///respective string particle (e.g. a colour string). In order to revert
    ///these step these particles with IDs 91-94 are searched for and their
    ///production vertices are removed from the event. 
    void RemovePythiaFragmentation( HepMC::GenEvent& event );

};

} // end of namespace HepMC
