/*****************************************************************************/
/* This is a basic example illustrating the use of the HepMC Visual library  */
/* in a standalone application.                                              */
/*                                                                           */
/* To run this example:                                                      */
/*  - Make sure ROOT version >= 5.10 is installed                            */
/*  - Make sure HepMC and HepMCVisual libraries are compiled and installed   */
/*    NOTE: if you didn't install in the default directory, you may need to  */
/*    modify the Makefile                                                    */
/*  - Make sure the directories holding the HepMC and HepMCVisual libraries  */
/*    are in your LD_LIBRARY_PATH                                            */
/*  - From the command line run the following commands:                      */
/*    > gmake basic                                                          */
/*    > ./basic                                                              */
/*                                                                           */
/* In case of problems, please write an email to:                            */
/*    hepmcvisual at projects.hepforge.org                                   */
/*                                                                           */
/*                                           Enjoy, Sebastian                */
/*****************************************************************************/

#include <TROOT.h>
#include <TApplication.h>
#include <TCanvas.h>
#include <TSystem.h>
#include <Riostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <TArrow.h>

#include "HepMCVisual/HepMCVisualElements.h"
#include "HepMC/IO_PDG_ParticleDataTable.h"

using namespace HepMC;

int main( int argc, char** argv ){

   /**
    * Create a basic root application
    */
   TApplication theApp("App", &argc, argv);
 
   if (gROOT->IsBatch()) {
      fprintf(stderr, "%s: cannot run in batch mode\n", argv[0]);
      return 1;
   }
 
   // Create a canvas to display the event
   TCanvas* chepmc = new TCanvas("chepmc","HepMCVisual Demo",900,600);

   /**
    * Construct a primitive HepMC event - this step can be left out if you have
    * another source of HepMC event records
    */
   GenVertex   * v0 = new GenVertex();
   GenVertex   * v1 = new GenVertex();
   GenVertex   * v2 = new GenVertex();
   GenVertex   * v3 = new GenVertex();

   // Z->e+e- with 90deg opening angle in lab-frame
   double m_Z = 91.187e3;
   double m_e = 0.511;
   double p_e = 0.5*sqrt(m_Z*m_Z - 4*m_e*m_e);
   double E_e = sqrt(0.5*m_Z*m_Z-m_e*m_e);
   GenParticle * p1 = new GenParticle(HepFourVector(0,0,2*p_e,2*E_e),23);
   GenParticle * p2 = new GenParticle(HepFourVector(0,p_e,p_e,E_e),11);
   GenParticle * p3 = new GenParticle(HepFourVector(0,-p_e,p_e,E_e),-11);

   v0->add_particle_out(p1);
   v1->add_particle_in(p1);
   v1->add_particle_out(p2); v2->add_particle_in(p2);
   v1->add_particle_out(p3); v3->add_particle_in(p3);

   GenEvent* evt = new GenEvent(1,0,0);
   evt->add_vertex(v0);
   evt->add_vertex(v1);
   evt->add_vertex(v2);
   evt->add_vertex(v3);
   evt->set_signal_process_vertex(v1);

   /**
    * Load a PDG table - this is necessary to display particle properties
    */
   IO_PDG_ParticleDataTable iopdt("PDG_table_2008.mc");
   ParticleDataTable* pdt = iopdt.read_particle_data_table();
   pdt->make_antiparticles_from_particles();

   /**
    * Finally visualize the event
    */
   VisualEvent* VisEvt = Visualize(evt);
   VisEvt->SetPDT(pdt);
   VisEvt->Draw();

   /**
    * Add some instructions
    */
  TLatex* l=new TLatex(.6,.6,"Double-click to start");
  l->SetTextSize(0.03); l->SetTextColor(kGray+2); l->Draw();
  TArrow* a =new TArrow(.59,.59,.515,.515,.02,"|>");
  a->SetLineColor(kGray+2); a->SetFillColor(kGray+2); a->Draw();
  l=new TLatex(.1,.8,"File->Quit to exit");
  l->SetTextSize(0.03); l->SetTextColor(kGray+2); l->Draw();
  a =new TArrow(.1,.83,.02,.99,.02,"|>");
  a->SetLineColor(kGray+2); a->SetFillColor(kGray+2); a->Draw();


   //Run the application (i.e. make the canvas react on user events)
   theApp.Run();
 
   return 0;
}
